@@ -1866,4 +1866,16 @@ public class IndexCPPTemplateResolutionTest extends IndexBindingResolutionTestBa
 		IBinding[] friends = ct.getFriends();
 		assertEquals(0, friends.length); // not yet supported
 	}
+
+	//	template<class T> struct C1 {
+	//	    typedef int iterator;
+	//	    iterator m1();
+	//	};
+
+	//	template<class T> typename C1<T>::iterator C1<T>::m1() {
+	//		return 0;
+	//	}
+	public void testUsageOfClassTemplateOutsideOfClassBody_357320() throws Exception {
+		getBindingFromASTName("m1", 0, ICPPMethod.class);
+	}
 }
