@@ -35,7 +35,6 @@ import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
 import org.eclipse.swt.widgets.Button;
@@ -119,6 +118,7 @@ public abstract class AbstractLangsListTab extends AbstractCPropertyTab {
 		};
 
 	private static final Comparator<Object> comp = CDTListComparator.getInstance();
+	private static String selectedLanguage;
 
 	private final static Image IMG_FOLDER = CDTSharedImages.getImage(CDTSharedImages.IMG_OBJS_FOLDER);
 	private final static Image IMG_INCLUDES_FOLDER = CDTSharedImages.getImage(CDTSharedImages.IMG_OBJS_INCLUDES_FOLDER);
@@ -291,6 +291,7 @@ public abstract class AbstractLangsListTab extends AbstractCPropertyTab {
 					ICLanguageSetting langSetting = (ICLanguageSetting) items[0].getData();
 					if (langSetting != null) {
 						lang = langSetting;
+						selectedLanguage = getLanguageName(lang);
 						update();
 					}
 				}
@@ -372,40 +373,47 @@ public abstract class AbstractLangsListTab extends AbstractCPropertyTab {
 		if (rcDes == null || !canBeVisible()) return;
 		updateExport();
 		langTree.removeAll();
-		TreeItem firstItem = null;
+		TreeItem selectedItem = null;
 		ls = getLangSetting(rcDes);
 		if (ls != null) {
 			Arrays.sort(ls, CDTListComparator.getInstance());
 			for (ICLanguageSetting langSetting : ls) {
 				if ((langSetting.getSupportedEntryKinds() & getKind()) != 0) {
 					TreeItem t = new TreeItem(langTree, SWT.NONE);
-					String langId = langSetting.getLanguageId();
-					if (langId != null && !langId.equals(EMPTY_STR)) {
-						// Bug #178033: get language name via LangManager.
-						ILanguageDescriptor langDes = LanguageManager.getInstance().getLanguageDescriptor(langId);
-						if (langDes == null)
-							langId = null;
-						else
-							langId = langDes.getName();
-					}
-					if (langId == null || langId.equals(EMPTY_STR))
-						langId = langSetting.getName();
+					String langId = getLanguageName(langSetting);
 					t.setText(0, langId);
 					t.setData(langSetting);
-					if (firstItem == null) {
-						firstItem = t;
+					if (selectedItem == null
+							|| (selectedLanguage != null && selectedLanguage.equals(langId))) {
+						selectedItem = t;
 						lang = langSetting;
 					}
 				}
 			}
 
-			if (firstItem != null && table != null) {
-				langTree.setSelection(firstItem);
+			if (selectedItem != null && table != null) {
+				langTree.setSelection(selectedItem);
 			}
 		}
 		update();
 	}
 
+	private String getLanguageName(ICLanguageSetting langSetting) {
+		String langId = langSetting.getLanguageId();
+		String langName = null;
+		if (langId != null && langId.length() != 0) {
+			// Bug #178033: get language name via LangManager.
+			ILanguageDescriptor langDes = LanguageManager.getInstance().getLanguageDescriptor(langId);
+			if (langDes != null)
+				langName = langDes.getName();
+		}
+		if (langName == null || langName.length() == 0)
+			langName = langSetting.getName();
+		if (langName == null || langName.length() == 0)
+			langName = langId;
+		return langName;
+	}
+
 	private void updateExport() {
 		exported = new ArrayList<ICSettingEntry>();
 		ICExternalSetting[] extSettings = getResDesc().getConfiguration().getExternalSettings();
