@@ -17,6 +17,7 @@
 package org.eclipse.cdt.dsf.mi.service;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Hashtable;
@@ -845,9 +846,9 @@ public class MIBreakpointsManager extends AbstractDsfService implements IBreakpo
         final Map<ICBreakpoint, Set<String>> threadsIDs = fBreakpointThreads.get(dmc);
         assert threadsIDs != null;
 
-        // Minimal validation
-        if (!platformBPs.containsKey(breakpoint)) {
-            rm.setStatus(new Status(IStatus.ERROR, GdbPlugin.PLUGIN_ID, INTERNAL_ERROR, UNKNOWN_BREAKPOINT, null));
+        boolean filtered = isBreakpointEntirelyFiltered(dmc, breakpoint);
+        
+        if (filtered && !platformBPs.containsKey(breakpoint)) {
             rm.done();
             return;
         }
@@ -860,7 +861,7 @@ public class MIBreakpointsManager extends AbstractDsfService implements IBreakpo
             // Note that Tracepoints are not affected by "skip-all"
         	boolean bpEnabled = attributes.get(ICBreakpoint.ENABLED).equals(true) && 
         						(breakpoint instanceof ICTracepoint || fBreakpointManager.isEnabled());
-        	if (bpEnabled) {
+        	if (!filtered && bpEnabled) {
                 attributes.put(ATTR_DEBUGGER_PATH, NULL_STRING);
                 attributes.put(ATTR_THREAD_FILTER, extractThreads(dmc, breakpoint));
                 attributes.put(ATTR_THREAD_ID, NULL_STRING);
@@ -876,6 +877,11 @@ public class MIBreakpointsManager extends AbstractDsfService implements IBreakpo
         	}
        		return;
         }
+        
+        if (filtered) {
+        	uninstallBreakpoint(dmc, breakpoint, rm );
+        	return;
+        }
 
         // Get the original breakpoint attributes
         final Map<String,Object> original_attributes = platformBPs.get(breakpoint);
@@ -975,8 +981,11 @@ public class MIBreakpointsManager extends AbstractDsfService implements IBreakpo
                 protected void handleSuccess() {
                     // Get the list of new back-end breakpoints contexts
                     newTargetBPs.addAll(getData());
+                    for (IBreakpointDMContext newRef : newTargetBPs)
+                    	targetBPs.put(newRef, breakpoint);
                     threadsIDs.put(breakpoint, newThreads);
                     for (final IBreakpointDMContext ref : oldTargetBPs) {
+                    	targetBPs.remove(ref);
                     	decrementInstallCount(ref, breakpoint, // A tad early but it should work...
                     			              new RequestMonitor(getExecutor(), removeRM) {
                     		@Override
@@ -1438,26 +1447,55 @@ public class MIBreakpointsManager extends AbstractDsfService implements IBreakpo
      */
     private void clearBreakpointStatus(final ICBreakpoint[] bps, final IBreakpointsTargetDMContext ctx)
     {
+        IWorkspaceRunnable wr = new IWorkspaceRunnable() {
+            public void run(IProgressMonitor monitor) throws CoreException {
+            	// For every platform breakpoint that has at least one target breakpoint installed
+            	// we must decrement the install count, for every target breakpoint.
+            	// Note that we cannot simply call resetInstallCount() because another
+            	// launch may be using the same platform breakpoint.
+            	Map<ICBreakpoint, Vector<IBreakpointDMContext>> breakpoints = fBreakpointIDs.get(ctx);
+            	for (ICBreakpoint breakpoint : breakpoints.keySet()) {
+            		Vector<IBreakpointDMContext> targetBps = breakpoints.get(breakpoint);
+            		for (IBreakpointDMContext targetBp : targetBps) {
+                        decrementInstallCount(targetBp, breakpoint, new RequestMonitor(getExecutor(), null));
+            		}
+                }
+            }
+        };
+
+        // Create the scheduling rule to clear all bp planted.
+        ISchedulingRule rule = null;
+        List<ISchedulingRule> markerRules = new ArrayList<ISchedulingRule>();
+        for (ICBreakpoint bp : bps) {
+            IMarker marker = bp.getMarker();
+            if (marker != null) {
+                ISchedulingRule markerRule =
+                    ResourcesPlugin.getWorkspace().getRuleFactory().markerRule(
+                            marker.getResource());
+                if (markerRule == null) {
+                    markerRules = null;
+                    break;
+                } else {
+                    markerRules.add(markerRule);
+                }
+            }
+        }
+        if (markerRules != null) {
+            rule = MultiRule.combine(markerRules.toArray(new ISchedulingRule[markerRules.size()]));
+        }
+
+        try {
+        	// Will run the workspace runnable on the current thread, which
+        	// is the DSF executor.
+            ResourcesPlugin.getWorkspace().run(wr, rule, 0, null);
+        } catch (CoreException e) {
+        	GdbPlugin.getDefault().getLog().log(e.getStatus());
+        }
+
         new Job("Clear Breakpoints Status") { //$NON-NLS-1$
             @Override
             protected IStatus run(IProgressMonitor monitor) {
-                IWorkspaceRunnable wr = new IWorkspaceRunnable() {
-                    public void run(IProgressMonitor monitor) throws CoreException {
-                    	// For every platform breakpoint that has at least one target breakpoint installed
-                    	// we must decrement the install count, for every target breakpoint.
-                    	// Note that we cannot simply call resetInstallCount() because another
-                    	// launch may be using the same platform breakpoint.
-                    	Map<ICBreakpoint, Vector<IBreakpointDMContext>> breakpoints = fBreakpointIDs.get(ctx);
-                    	for (ICBreakpoint breakpoint : breakpoints.keySet()) {
-                    		Vector<IBreakpointDMContext> targetBps = breakpoints.get(breakpoint);
-                    		for (IBreakpointDMContext targetBp : targetBps) {
-                                decrementInstallCount(targetBp, breakpoint, new RequestMonitor(getExecutor(), null));
-                    		}
-                        }
-                    }
-                };
-
-                // First clear any problem markers
+                // Clear any problem markers
                 for (IMarker marker : fBreakpointMarkerProblems.values()) {
                 	if (marker != null) {
                 		try {
@@ -1468,32 +1506,6 @@ public class MIBreakpointsManager extends AbstractDsfService implements IBreakpo
                 }
                 fBreakpointMarkerProblems.clear();
                 
-                // Create the scheduling rule to clear all bp planted.
-                ISchedulingRule rule = null;
-                List<ISchedulingRule> markerRules = new ArrayList<ISchedulingRule>();
-                for (ICBreakpoint bp : bps) {
-                    IMarker marker = bp.getMarker();
-                    if (marker != null) {
-                        ISchedulingRule markerRule =
-                            ResourcesPlugin.getWorkspace().getRuleFactory().markerRule(
-                                    marker.getResource());
-                        if (markerRule == null) {
-                            markerRules = null;
-                            break;
-                        } else {
-                            markerRules.add(markerRule);
-                        }
-                    }
-                }
-                if (markerRules != null) {
-                    rule = MultiRule.combine(markerRules.toArray(new ISchedulingRule[markerRules.size()]));
-                }
-
-                try {
-                    ResourcesPlugin.getWorkspace().run(wr, rule, 0, null);
-                } catch (CoreException e) {
-                    return e.getStatus();
-                }
                 return Status.OK_STATUS;
             }
         }.schedule();
@@ -1883,4 +1895,18 @@ public class MIBreakpointsManager extends AbstractDsfService implements IBreakpo
         return !(breakpoint instanceof ICWatchpoint);
     }
 
+    /**
+     * Returns whether the breakpoint is filtered for given target.
+     */
+    private boolean isBreakpointEntirelyFiltered(IBreakpointsTargetDMContext dmc, ICBreakpoint breakpoint) {
+		IContainerDMContext currentDmc = DMContexts.getAncestorOfType(dmc, IContainerDMContext.class);
+    	try {
+			IContainerDMContext[] targetDmcs = getFilterExtension(breakpoint).getTargetFilters();
+			if (Arrays.asList(targetDmcs).contains(currentDmc))
+				return false;
+		}
+		catch(CoreException e) {
+		}
+    	return true;
+    }
 }
