@@ -8,6 +8,7 @@
  * Contributors:
  *     Wind River Systems - initial API and implementation
  *     Ericsson	AB		  - Modified for handling of multiple threads
+ *     Vladimir Prus (Mentor Graphics) - Add proper stop reason for step return (Bug 362274) 
  *******************************************************************************/
 package org.eclipse.cdt.dsf.mi.service;
 
@@ -51,6 +52,7 @@ import org.eclipse.cdt.dsf.mi.service.command.events.MIBreakpointHitEvent;
 import org.eclipse.cdt.dsf.mi.service.command.events.MICatchpointHitEvent;
 import org.eclipse.cdt.dsf.mi.service.command.events.MIErrorEvent;
 import org.eclipse.cdt.dsf.mi.service.command.events.MIEvent;
+import org.eclipse.cdt.dsf.mi.service.command.events.MIFunctionFinishedEvent;
 import org.eclipse.cdt.dsf.mi.service.command.events.MIRunningEvent;
 import org.eclipse.cdt.dsf.mi.service.command.events.MISharedLibEvent;
 import org.eclipse.cdt.dsf.mi.service.command.events.MISignalEvent;
@@ -187,6 +189,8 @@ public class MIRunControl extends AbstractDsfService implements IMIRunControl, I
 				return StateChangeReason.BREAKPOINT;
 			} else if (getMIEvent() instanceof MISteppingRangeEvent) {
 				return StateChangeReason.STEP;
+			} else if (getMIEvent() instanceof MIFunctionFinishedEvent) {
+				return StateChangeReason.STEP;
 			} else if (getMIEvent() instanceof MISharedLibEvent) {
 				return StateChangeReason.SHAREDLIB;
 			}else if (getMIEvent() instanceof MISignalEvent) {
