@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2009, 2010 Ericsson and others.
+ * Copyright (c) 2010 Ericsson and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -8,18 +8,18 @@
  * Contributors:
  *     Ericsson - Initial Implementation
  *******************************************************************************/
-package org.eclipse.cdt.tests.dsf.gdb.tests.tests_6_7;
+package org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_2;
 
 import org.eclipse.cdt.tests.dsf.gdb.framework.BackgroundRunner;
 import org.eclipse.cdt.tests.dsf.gdb.tests.ITestConstants;
-import org.eclipse.cdt.tests.dsf.gdb.tests.MIBreakpointsTest;
+import org.eclipse.cdt.tests.dsf.gdb.tests.tests_7_1.MIBreakpointsTest_7_1;
 import org.junit.BeforeClass;
 import org.junit.runner.RunWith;
 
 @RunWith(BackgroundRunner.class)
-public class MIBreakpointsTest_6_7 extends MIBreakpointsTest {
+public class MIBreakpointsTest_7_2 extends MIBreakpointsTest_7_1 {
 	@BeforeClass
-    public static void beforeClassMethod_6_7() {
-		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_6_7);		
+	public static void beforeClassMethod_7_2() {
+		setGdbProgramNamesLaunchAttributes(ITestConstants.SUFFIX_GDB_7_2);		
 	}
 }
